@@ -1,7 +1,7 @@
 import tensorflow.keras as ks
 
 from kgcnn.layers.casting import ChangeTensorType, ChangeIndexing
-from kgcnn.layers.conv import SchNetInteraction
+from kgcnn.layers.interaction import SchNetInteraction
 from kgcnn.layers.keras import Dense
 from kgcnn.layers.mlp import MLP
 from kgcnn.layers.pooling import PoolingNodes
